/*******************************************************************************
* File Name: Filter_FIR_PM.c
* Version 2.20
*
* Description:
*  This file provides the power managaer API source code for the FILT component.
*
* Note:
*  
*******************************************************************************
* Copyright 2008-2013, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
********************************************************************************/

#include "Filter_FIR_PVT.h"

static Filter_FIR_backupStruct  Filter_FIR_backup = 
{
    Filter_FIR_DISABLED,
    Filter_FIR_RUN_MASK,
    
};


/*******************************************************************************
* Function Name: Filter_FIR_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the current user configuration.
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Filter_FIR_backup:  This global structure variable is used to store 
*  configuration registers which are non retention whenever user wants to go 
*  sleep mode by calling Sleep() API.
*
*******************************************************************************/
void Filter_FIR_SaveConfig(void) 
{
    Filter_FIR_backup.cr = Filter_FIR_CR_REG;
    Filter_FIR_backup.sr = Filter_FIR_SR_REG;
    Filter_FIR_backup.sema = Filter_FIR_SEMA_REG;

    /* Put DFB RAM on the bus */
    Filter_FIR_RAM_DIR_REG = Filter_FIR_RAM_DIR_BUS;

    /* Save the ACU RAM as this is not retension */
    (void)memcpy(Filter_FIR_backup.saveAcu,
        Filter_FIR_ACU_RAM, Filter_FIR_ACU_RAM_SIZE); 

    /* Take DFB RAM off the bus */
    Filter_FIR_RAM_DIR_REG = Filter_FIR_RAM_DIR_DFB;
}


/*******************************************************************************
* Function Name: Filter_FIR_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the current user configuration.
*
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Filter_FIR_backup:  This global structure variable is used to restore 
*  configuration registers which are non retention whenever user wants to switch 
*  to active power mode by calling Wakeup() API.
*
*******************************************************************************/
void Filter_FIR_RestoreConfig(void) 
{
    Filter_FIR_CR_REG = Filter_FIR_backup.cr;
    Filter_FIR_SR_REG = Filter_FIR_backup.sr;
    Filter_FIR_SEMA_REG = Filter_FIR_backup.sema;
    
    /* Power on DFB before initializing the RAMs */
    Filter_FIR_PM_ACT_CFG_REG |= Filter_FIR_PM_ACT_MSK;

    /* Put DFB RAM on the bus */
    Filter_FIR_RAM_DIR_REG = Filter_FIR_RAM_DIR_BUS;

    /* Restore ACU RAM as this is not retension */
    (void)memcpy(Filter_FIR_ACU_RAM,
        Filter_FIR_backup.saveAcu, Filter_FIR_ACU_RAM_SIZE); 

    /* Take DFB RAM off the bus */
    Filter_FIR_RAM_DIR_REG = Filter_FIR_RAM_DIR_DFB;
}


/*******************************************************************************
* Function Name: Filter_FIR_Sleep
********************************************************************************
*
* Summary:
*  Disables block's operation and saves its configuration. Should be called 
*  just prior to entering sleep.
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Filter_FIR_backup:  The structure field 'enableState' is modified 
*  depending on the enable state of the block before entering to sleep mode.
*
*******************************************************************************/
void Filter_FIR_Sleep(void) 
{
    /* Save Filter enable state */
    if(Filter_FIR_PM_ACT_MSK == (Filter_FIR_PM_ACT_CFG_REG & Filter_FIR_PM_ACT_MSK))
    {
        /* Component is enabled */
        Filter_FIR_backup.enableState = Filter_FIR_ENABLED;
    }
    else
    {
        /* Component is disabled */
        Filter_FIR_backup.enableState = Filter_FIR_DISABLED;
    }

    /* Save the configuration */
    Filter_FIR_SaveConfig();
    
    /* Stop the component */
    Filter_FIR_Stop();
}


/*******************************************************************************
* Function Name: Filter_FIR_Wakeup
********************************************************************************
*
* Summary:
*  Enables block's operation and restores its configuration. Should be called
*  just after awaking from sleep.
*  
* Parameters:  
*  void
*
* Return: 
*  void
*
* Global variables:
*  Filter_FIR_backup:  The structure field 'enableState' is used to 
*  restore the enable state of block after wakeup from sleep mode.
*
*******************************************************************************/
void Filter_FIR_Wakeup(void) 
{
    /* Restore the configuration */
    Filter_FIR_RestoreConfig();
    
    /* Enable's the component operation */
    if(Filter_FIR_backup.enableState == Filter_FIR_ENABLED)
    {
        Filter_FIR_Enable();
    } /* Do nothing if component was disable before */
}


/* [] END OF FILE */
